##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import time

from AccessControl import ClassSecurityInfo
from AccessControl.class_init import InitializeClass
from Acquisition import Implicit
from BTrees.IIBTree import IIBTree
from BTrees.IOBTree import IOBTree
from BTrees.OOBTree import OOBTree
from Persistence import Persistent

from .EventLog import EventLog
from .EventLog import LogEntry
from .Utility import VersionControlError
from .ZopeVersion import ZopeVersion


MAX32 = int(2**31 - 1)


class VersionHistory(Implicit, Persistent):
    """A version history maintains the information about the changes
       to a particular version-controlled resource over time."""

    def __init__(self, history_id, object):
        # The _versions mapping maps version ids to version objects. All
        # of the actual version data is looked up there. The _labels
        # mapping maps labels to specific version ids. The _branches map
        # manages BranchInfo objects that maintain branch information.
        self._eventLog = EventLog()
        self._versions = OOBTree()
        self._branches = OOBTree()
        self._labels = OOBTree()
        self.createBranch('mainline', None)
        self.id = history_id

    security = ClassSecurityInfo()

    @security.public
    def getId(self):
        return self.id

    @security.private
    def addLogEntry(self, version_id, action, path=None, message=''):
        """Add a new log entry associated with this version history."""
        entry = LogEntry(version_id, action, path, message)
        self._eventLog.addEntry(entry)

    @security.private
    def getLogEntries(self):
        """Return a sequence of the log entries for this version history."""
        return self._eventLog.getEntries()

    @security.private
    def getLabels(self):
        return self._labels.keys()

    @security.private
    def labelVersion(self, version_id, label, force=0):
        """Associate a particular version in a version history with the
           given label, removing any existing association with that label
           if force is true, or raising an error if force is false and
           an association with the given label already exists."""
        current = self._labels.get(label)
        if current is not None:
            if current == version_id:
                return
            if not force:
                raise VersionControlError(
                    'The label %s is already associated with a version.' % (
                        label
                    ))
            del self._labels[label]
        self._labels[label] = version_id

    @security.private
    def createBranch(self, branch_id, version_id):
        """Create a new branch associated with the given branch_id. The
           new branch is rooted at the version named by version_id."""
        if branch_id in self._branches:
            raise VersionControlError(
                'Activity already exists: %s' % branch_id
            )
        branch = BranchInfo(branch_id, version_id)
        self._branches[branch_id] = branch
        return branch

    @security.private
    def createVersion(self, object, branch_id):
        """Create a new version in the line of descent named by the given
           branch_id, returning the newly created version object."""
        branch = self._branches.get(branch_id)
        if branch is None:
            branch = self.createBranch(branch_id, None)
        if branch.name != 'mainline':
            version_id = '%s.%d' % (branch.name, len(branch) + 1)
        else:
            version_id = '%d' % (len(branch) + 1)
        version = ZopeVersion(version_id, object)

        # Update the  predecessor, successor and branch relationships.
        # This is something of a hedge against the future. Versions will
        # always know enough to reconstruct their lineage without the help
        # of optimized data structures, which will make it easier to change
        # internals in the future if we need to.
        latest = branch.latest()
        if latest is not None:
            last = self._versions[latest]
            last.next = last.next + (version_id,)
            version.prev = latest

        # If the branch is not the mainline, store the branch name in the
        # version. Versions have 'mainline' as the default class attribute
        # which is the common case and saves a minor bit of storage space.
        if branch.name != 'mainline':
            version.branch = branch.name

        branch.append(version)
        self._versions[version_id] = version
        # Call saveState() only after version has been linked into the
        # database, ensuring it goes into the correct database.
        version.saveState(object)
        return version.__of__(self)

    @security.private
    def hasVersionId(self, version_id):
        """Return true if history contains a version with the given id."""
        return version_id in self._versions

    @security.private
    def isLatestVersion(self, version_id, branch_id):
        """Return true if version id is the latest in its branch."""
        branch = self._branches[branch_id]
        return version_id == branch.latest()

    @security.private
    def getLatestVersion(self, branch_id):
        """Return the latest version object within the given branch, or
           None if the branch contains no versions."""
        branch = self._branches[branch_id]
        version = self._versions[branch.latest()]
        return version.__of__(self)

    @security.private
    def findBranchId(self, version_id):
        """Given a version id, return the id of the branch of the version.
           Note that we cheat, since we can find this out from the id."""
        parts = version_id.split('.')
        if len(parts) > 1:
            return parts[-2]
        return 'mainline'

    @security.private
    def getVersionById(self, version_id):
        """Return the version object named by the given version id, or
           raise a VersionControlError if the version is not found."""
        version = self._versions.get(version_id)
        if version is None:
            raise VersionControlError(
                'Unknown version id: %s' % version_id
            )
        return version.__of__(self)

    @security.private
    def getVersionByLabel(self, label):
        """Return the version associated with the given label, or None
           if no version matches the given label."""
        version_id = self._labels.get(label)
        version = self._versions.get(version_id)
        if version is None:
            return None
        return version.__of__(self)

    @security.private
    def getVersionByDate(self, branch_id, timestamp):
        """Return the last version committed in the given branch on or
           before the given time value. The timestamp should be a float
           (time.time format) value in UTC."""
        branch = self._branches[branch_id]
        tvalue = int(timestamp / 60.0)
        while True:
            # Try to find a version with a commit date <= the given time
            # using the timestamp index in the branch information.
            if branch.m_order:
                try:
                    match = branch.m_date.maxKey(tvalue)
                    match = branch.m_order[branch.m_date[match]]
                    return self._versions[match].__of__(self)
                except ValueError:
                    pass

            # If we've run out of lineage without finding a version with
            # a commit date <= the given timestamp, we return None. It is
            # up to the caller to decide what to do in this situation.
            if branch.root is None:
                return None

            # If the branch has a root (a version in another branch), then
            # we check the root and do it again with the ancestor branch.
            rootver = self._versions[branch.root]
            if int(rootver.date_created / 60.0) < tvalue:
                return rootver.__of__(self)
            branch = self._branches[rootver.branch]

    @security.private
    def getVersionIds(self, branch_id=None):
        """Return a sequence of version ids for the versions in this
           version history. If a branch_id is given, only version ids
           from that branch will be returned. Note that the sequence
           of ids returned does not include the id of the branch root."""
        if branch_id is not None:
            return self._branches[branch_id].versionIds()
        return self._versions.keys()


InitializeClass(VersionHistory)


class BranchInfo(Implicit, Persistent):
    """A utility class to hold branch (line-of-descent) information. It
       maintains the name of the branch, the version id of the root of
       the branch and indices to allow for efficient lookups."""

    security = ClassSecurityInfo()

    def __init__(self, name, root):
        # m_order maintains a newest-first mapping of int -> version id.
        # m_date maintains a mapping of a packed date (int # of minutes
        # since the epoch) to a lookup key in m_order. The two structures
        # are separate because we only support minute precision for date
        # lookups (and multiple versions could be added in a minute).
        self.date_created = time.time()
        self.m_order = IOBTree()
        self.m_date = IIBTree()
        self.name = name
        self.root = root

    @security.public
    def getId(self):
        """Return the name of the object as string."""
        return self.name

    @security.private
    def append(self, version):
        """Append a version to the branch information. Note that this
           does not store the actual version, but metadata about the
           version to support ordering and date lookups."""
        if len(self.m_order):
            key = self.m_order.minKey() - 1
        else:
            key = MAX32
        self.m_order[key] = version.id
        timestamp = int(version.date_created / 60.0)
        self.m_date[timestamp] = key

    @security.private
    def versionIds(self):
        """Return a newest-first sequence of version ids in the branch."""
        return self.m_order.values()

    @security.private
    def latest(self):
        """Return the version id of the latest version in the branch."""
        mapping = self.m_order
        if not len(mapping):
            return self.root
        return mapping[mapping.keys()[0]]

    def __len__(self):
        return len(self.m_order)


InitializeClass(BranchInfo)



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import AccessControl
import OFS
from AccessControl.class_init import InitializeClass
from App.special_dtml import DTMLFile
from OFS.role import RoleManager

from . import Repository
from .SequenceWrapper import SequenceWrapper


class ZopeRepository(
    Repository.Repository,
    RoleManager,
    OFS.SimpleItem.Item
):
    """The ZopeRepository class builds on the core Repository implementation
       to provide the Zope management interface and other product trappings."""

    security = AccessControl.ClassSecurityInfo()

    meta_type = 'Repository'

    manage_options = (
        ({'label': 'Contents', 'action': 'manage_main',
          'help': ('ZopeVersionControl', 'Repository-Manage.stx')},
         {'label': 'Properties', 'action': 'manage_properties_form',
          'help': ('ZopeVersionControl', 'Repository-Properties.stx')},
         ) +
        RoleManager.manage_options +
        OFS.SimpleItem.Item.manage_options
    )

    security.declareProtected('View management screens', 'manage_main')
    manage_main = DTMLFile('dtml/RepositoryManageMain', globals())
    manage_main._setName('manage_main')
    manage = manage_main

    def __init__(self, id=None, title=''):
        Repository.Repository.__init__(self)
        if id is not None:
            self._setId(id)
        self.title = title

    security.declareProtected(
        'View management screens', 'manage_properties_form'
    )
    manage_properties_form = DTMLFile('dtml/RepositoryProperties', globals())

    @security.protected('Manage repositories')
    def manage_edit(self, title='', REQUEST=None):
        """Change object properties."""
        self.title = title
        if REQUEST is not None:
            message = "Saved changes."
            return self.manage_properties_form(
                self, REQUEST, manage_tabs_message=message
            )

    def __getitem__(self, name):
        history = self._histories.get(name)
        if history is not None:
            return history.__of__(self)
        raise KeyError(name)

    @security.private
    def objectIds(self, spec=None):
        return SequenceWrapper(self, self._histories.keys())

    @security.private
    def objectValues(self, spec=None):
        return SequenceWrapper(self, self._histories.values())

    @security.private
    def objectItems(self, spec=None):
        return SequenceWrapper(self, self._histories.items(), 1)


InitializeClass(ZopeRepository)


def addRepository(self, id, title='', REQUEST=None):
    """Zope object constructor function."""
    object = ZopeRepository(title=title)
    object._setId(id)
    self._setObject(id, object)
    object = self._getOb(id)
    if REQUEST is not None:
        try:
            url = self.DestinationURL()
        except BaseException:
            url = REQUEST['URL1']
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)
    return


addRepositoryForm = DTMLFile('dtml/RepositoryAddForm', globals())


constructors = (
    ('addRepositoryForm', addRepositoryForm),
    ('addRepository', addRepository),
)



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import time

from AccessControl import ClassSecurityInfo
from AccessControl.class_init import InitializeClass
from BTrees.IOBTree import IOBTree
from Persistence import Persistent

from .Utility import _findUserId


MAX32 = int(2**31 - 1)


class EventLog(Persistent):
    """An EventLog encapsulates a collection of log entries."""

    def __init__(self):
        self._data = IOBTree()

    security = ClassSecurityInfo()

    @security.private
    def addEntry(self, entry):
        """Add a new log entry."""
        if len(self._data):
            key = self._data.minKey() - 1
        else:
            key = MAX32
        self._data[key] = entry

    @security.private
    def getEntries(self):
        """Return a sequence of log entries."""
        return self._data.values()

    def __len__(self):
        return len(self._data)

    def __nonzero__(self):
        return len(self._data) > 0


InitializeClass(EventLog)


class LogEntry(Persistent):
    """A LogEntry contains audit information about a version control
       operation. Actions that cause audit records to be created include
       checkout and checkin. Log entry information can be read (but
       not changed) by restricted code."""

    # These action constants represent the possible auditable actions.
    ACTION_CHECKOUT = 0
    ACTION_CHECKIN = 1
    ACTION_UNCHECKOUT = 2
    ACTION_UPDATE = 3

    def __init__(self, version_id, action, path=None, message=''):
        self.timestamp = time.time()
        self.version_id = version_id
        self.action = action
        self.message = message
        self.user_id = _findUserId()
        self.path = path


InitializeClass(LogEntry)



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import AccessControl
import OFS
from AccessControl.class_init import InitializeClass
from App.special_dtml import DTMLFile
from OFS.role import RoleManager

from . import Version


class ZopeVersion(Version.Version, RoleManager, OFS.SimpleItem.Item):
    """The ZopeVersion class builds on the core Version class to provide
       the Zope management interface and other product trappings."""

    security = AccessControl.ClassSecurityInfo()
    security.setDefaultAccess('deny')

    meta_type = 'Version'

    manage_options = (
        ({'label': 'Information', 'action': 'manage_main',
          'help': ('ZopeVersionControl', 'Version-Manage.stx')},
         {'label': 'Properties', 'action': 'manage_properties_form',
          'help': ('ZopeVersionControl', 'Version-Properties.stx')},
         ) +
        RoleManager.manage_options +
        OFS.SimpleItem.Item.manage_options
    )

    icon = 'misc_/ZopeVersionControl/Version.gif'

    security.declareProtected('View management screens', 'manage_main')
    manage_main = DTMLFile('dtml/VersionManageMain', globals())
    manage_main._setName('manage_main')
    manage = manage_main

    security.declareProtected(
        'View management screens', 'manage_properties_form'
    )
    manage_properties_form = DTMLFile('dtml/VersionProperties', globals())

    @security.protected('Manage repositories')
    def manage_edit(self, REQUEST=None):
        """Change object properties."""
        if REQUEST is not None:
            message = "Saved changes."
            return self.manage_properties_form(
                self, REQUEST, manage_tabs_message=message

            )


InitializeClass(ZopeVersion)



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
from zope.interface import Interface


class IVersionControl(Interface):
    """The version control interface serves as the main API for version
       control operations. The interface hides most of the details of
       version data storage and retrieval.

       In Zope 3, the version control interface will probably be implemented
       by a version control service. In the meantime, it may be implemented
       directly by repository implementations (or other things, like CMF
       tools).

       The goal of this version of the version control interface is to
       support simple linear versioning with support for labelled versions.
       Future versions or extensions of this interface will likely support
       more advanced version control features such as concurrent lines of
       descent (activities) and collection versioning."""

    def isAVersionableResource(object):
        """
        Returns true if the given object is a versionable resource.

        Permission: public
        """

    def isUnderVersionControl(object):
        """
        Returns true if the given object is under version control.

        Permission: public
        """

    def isResourceUpToDate(object, require_branch=0):
        """
        Returns true if a resource is based on the latest version. Note
        that the latest version is in the context of any activity (branch).

        If the require_branch flag is true, this method returns false if
        the resource is updated to a particular version, label, or date.
        Useful for determining whether a call to checkoutResource()
        will succeed.

        Permission: public
        """

    def isResourceChanged(object):
        """
        Return true if the state of a resource has changed in a transaction
        *after* the version bookkeeping was saved. Note that this method is
        not appropriate for detecting changes within a transaction!

        Permission: public
        """

    def getVersionInfo(object):
        """
        Return the VersionInfo associated with the given object. The
        VersionInfo object contains version control bookkeeping information.
        If the object is not under version control, a VersionControlError
        will be raised.

        Permission: public
        """

    def applyVersionControl(object, message=None):
        """
        Place the given object under version control. A VersionControlError
        will be raised if the object is already under version control.

        After being placed under version control, the resource is logically
        in the 'checked-in' state.

        If no message is passed the 'Initial checkin.' message string is
        written as the message log entry.

        Permission: Use version control
        """

    def checkoutResource(object):
        """
        Put the given version-controlled object into the 'checked-out'
        state, allowing changes to be made to the object. If the object is
        not under version control or the object is already checked out, a
        VersionControlError will be raised.

        Permission: Use version control
        """

    def checkinResource(object, message=''):
        """
        Check-in (create a new version) of the given object, updating the
        state and bookkeeping information of the given object. The optional
        message should describe the changes being committed. If the object
        is not under version control or is already in the checked-in state,
        a VersionControlError will be raised.

        Permission: Use version control
        """

    def uncheckoutResource(object):
        """
        Discard changes to the given object made since the last checkout.
        If the object is not under version control or is not checked out,
        a VersionControlError will be raised.
        """

    def updateResource(object, selector=None):
        """
        Update the state of the given object to that of a specific version
        of the object. The object must be in the checked-in state to be
        updated. The selector must be a string (version id, activity id,
        label or date) that is used to select a version from the version
        history.

        Permission: Use version control
        """

    def labelResource(object, label, force=None):
        """
        Associate the given resource with a label. If force is true, then
        any existing association with the given label will be removed and
        replaced with the new association. If force is false and there is
        an existing association with the given label, a VersionControlError
        will be raised.

        Permission: Use version control
        """

    def getVersionOfResource(history_id, selector):
        """
        Given a version history id and a version selector, return the
        object as of that version. Note that the returned object has no
        acquisition context. The selector must be a string (version id,
        activity id, label or date) that is used to select a version
        from the version history.

        Permission: Use version control
        """

    def getVersionIds(object):
        """
        Return a sequence of the (string) version ids corresponding to the
        available versions of an object. This should be used by UI elements
        to populate version selection widgets, etc.

        Permission: Use version control
        """

    def getLabelsForResource(object):
        """
        Return a sequence of the (string) labels corresponding to the
        versions of the given object that have been associated with a
        label. This should be used by UI elements to populate version
        selection widgets, etc.

        Permission: Use version control
        """

    def getLogEntries(object):
        """
        Return a sequence of LogEntry objects (most recent first) that
        are associated with a version-controlled object.

        Permission: Use version control
        """


class IVersionInfo(Interface):
    """The IVersionInfo interface provides access to version control
       bookkeeping information. The fields provided by this interface
       are:

         timestamp - a float (time.time() format) value indicating the
         time that the bookkeeping information was created.

         history_id - the id of the version history related to the version
         controlled resource.

         version_id - the version id that the version controlled resource
         is based upon.

         status - an enumerated value indicating the status of the version
         controlled resource. This value is one of the VersionInfo class
         constants CHECKED_IN or CHECKED_OUT.

         sticky - sticky tag information used internally by the version
         control implementation.

         user_id - the id of the effective user at the time the bookkeeping
         information was created.
         """


class ILogEntry(Interface):
    """The ILogEntry interface provides access to the information in an
       audit log entry. The fields provided by this interface are:

         timestamp - a float (time.time() format) value indicating the
         time that the log entry was created.

         version_id - the version id of the version controlled resource
         related to the log entry.

         action - an enumerated value indicating the action that was taken.
         This value is one of the LogEntry class constants ACTION_CHECKOUT,
         ACTION_CHECKIN, ACTION_UNCHECKOUT, ACTION_UPDATE.

         message - a string message provided by the user at the time of the
         action. This string may be empty.

         user_id - the id of the user causing the audited action.

         path - the path to the object upon which the action was taken.

         """


class INonVersionedData(Interface):
    """Controls what parts of an object fall outside version control.

    Containerish objects implement this interface to allow the items they
    contain to be versioned independently of the container.
    """

    def listNonVersionedObjects():
        """Returns a list of subobjects that should not be pickled.

        The objects in the list must not be wrapped, because only the
        identity of the objects will be considered.  The version
        repository uses this method to avoid cloning subobjects that
        will soon be removed by removeNonVersionedData.
        """

    def removeNonVersionedData():
        """Removes the non-versioned data from this object.

        The version repository uses this method before storing an
        object in the version repository.
        """

    def getNonVersionedData():
        """Returns an opaque object containing the non-versioned data.

        The version repository uses this method before reverting an
        object to a revision.
        """

    def restoreNonVersionedData(dict):
        """Restores non-versioned data to this object.

        The version repository uses this method after reverting an
        object to a revision.
        """


IVersionedContainer = INonVersionedData



from setuptools import find_packages
from setuptools import setup


setup(
    name='Products.ZopeVersionControl',
    version='4.0',
    description="Zope Version Control",
    long_description=(open('README.rst').read() + "\n" +
                      open('CHANGES.rst').read()),
    classifiers=[
        'Development Status :: 6 - Mature',
        'Framework :: Zope :: 5',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    keywords='version control',
    license='ZPL 2.1',
    author='Zope Foundation and Contributors',
    author_email='zope-dev@zope.dev',
    url='https://github.com/zopefoundation/Products.ZopeVersionControl',
    project_urls={
        'Issue Tracker': ('https://github.com/zopefoundation/'
                          'Products.ZopeVersionControl/issues'),
        'Sources': ('https://github.com/zopefoundation/'
                    'Products.ZopeVersionControl'),
    },
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['Products'],
    include_package_data=True,
    zip_safe=False,
    python_requires='>=3.7',
    install_requires=[
        'setuptools',
        'zope.interface',
        'Acquisition',
        'DateTime',
        'transaction',
        'ZODB',
        'Zope',
    ],
    entry_points={
        'zodbupdate.decode': [
            'decodes = Products.ZopeVersionControl:zodbupdate_decode_dict',
        ],
    },
)



##############################################################################
#
# Copyright (c) 2003 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this
# distribution.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Run all Zope Version Control tests."""

import unittest

from Products.ZopeVersionControl.tests import testVersionControl


try:
    from Products import References
    del References
except ImportError:
    # References product is not available
    testReferenceVersioning = None
else:
    # References product is available
    from Products.ZopeVersionControl.tests import testReferenceVersioning


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(testVersionControl.test_suite())
    if testReferenceVersioning is not None:
        suite.addTest(testReferenceVersioning.test_suite())
    return suite
